#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const lodash_1 = require("lodash");
const fs_1 = __importDefault(require("fs"));
const yargs_1 = __importDefault(require("yargs"));
const glob_1 = __importDefault(require("glob"));
const typescript_1 = __importDefault(require("typescript"));
const typeResolver_1 = require("../core/Controller/typeResolver");
const lodash_2 = require("lodash");
(function run() {
    const basePath = getBasePath();
    const baseSwagger = getBaseSwagger();
    const { swaggerComponents } = require(basePath + 'dist/Backend/app.js');
    validateBaseSwagger(baseSwagger);
    const responseSchemas = getResponseSchemas();
    baseSwagger.paths = lodash_2.merge(swaggerComponents, responseSchemas || {});
    for (const path of Object.keys(baseSwagger.paths)) {
        for (const method of Object.keys(baseSwagger.paths[path])) {
            if (!baseSwagger.paths[path][method]['responses'] ||
                !Object.keys(baseSwagger.paths[path][method]['responses']).length) {
                baseSwagger.paths[path][method]['responses'] = {
                    default: {
                        description: 'No response specified',
                    },
                };
            }
        }
    }
    const args = yargs_1.default(process.argv).argv;
    const outputFileName = args && args.output ? args.output : 'oas.json';
    const outputFilePath = basePath + outputFileName;
    fs_1.default.writeFileSync(outputFilePath, JSON.stringify(baseSwagger));
})();
function getBasePath() {
    const currentModuleRelativePath = lodash_1.get(require, 'main.filename');
    if (!currentModuleRelativePath)
        throw new Error('Unable to locate module path');
    const currentModuleAbsolutePath = path_1.default.dirname(currentModuleRelativePath);
    const pathComponents = currentModuleAbsolutePath.split('node_modules');
    const basePath = pathComponents.slice(0, pathComponents.length - 1).join('node_modules');
    return basePath;
}
function getBaseSwagger() {
    const args = yargs_1.default(process.argv).argv;
    const configFileName = args && args.config ? args.config : 'dsapidoc.json';
    const configFile = getBasePath() + configFileName;
    const config = JSON.parse(fs_1.default.readFileSync(configFile).toString());
    return config;
}
function validateBaseSwagger(_baseSwagger) { }
function getResponseSchemas() {
    const files = getFiles();
    const compilerOptions = {};
    const program = typescript_1.default.createProgram(files, compilerOptions);
    const nodes = program.getSourceFiles().flatMap((sf) => {
        const nodes = [];
        typescript_1.default.forEachChild(sf, (node) => {
            if (typescript_1.default.isModuleDeclaration(node)) {
                if ((node.flags & typescript_1.default.NodeFlags.Namespace) === 0 && node.body && typescript_1.default.isModuleBlock(node.body)) {
                    node.body.statements.forEach((statement) => {
                        nodes.push(statement);
                    });
                    return;
                }
            }
            nodes.push(node);
        });
        return nodes;
    });
    // Refactor this global to provide as parameter to type resolver
    // @ts-ignore
    global.allNodes = nodes;
    const responses = {};
    for (const node of nodes) {
        if (node.kind === typescript_1.default.SyntaxKind.ClassDeclaration && getDecorators(node, 'Controller').length > 0) {
            const decorator = getDecorators(node, 'Controller')[0];
            const value = getDecoratorValues(decorator);
            const baseRoute = value[0];
            const methods = node.members.filter((m) => m.kind === typescript_1.default.SyntaxKind.MethodDeclaration);
            for (const method of methods) {
                ['Get', 'Post', 'Put', 'Delete'].forEach((e) => {
                    const decorator = getDecorators(method, e)[0];
                    if (decorator) {
                        const value = getDecoratorValues(decorator);
                        const specificRoute = value[0] || '';
                        responses[baseRoute + specificRoute] = responses[baseRoute + specificRoute] || {};
                        responses[baseRoute + specificRoute][e.toLowerCase()] =
                            responses[baseRoute + specificRoute][e.toLowerCase()] || {};
                        responses[baseRoute + specificRoute][e.toLowerCase()]['responses'] =
                            responses[baseRoute + specificRoute][e.toLowerCase()]['responses'] || {};
                        const responseDecorators = getDecorators(method, 'Response');
                        for (const responseDEcorator of responseDecorators) {
                            const value = getDecoratorValues(responseDEcorator);
                            const code = value[0];
                            const description = value[1];
                            if (responseDEcorator.expression.typeArguments &&
                                responseDEcorator.expression.typeArguments[0]) {
                                const type = typeResolver_1.resolveType(responseDEcorator.expression.typeArguments[0]);
                                if (type.typeName === 'array') {
                                    responses[baseRoute + specificRoute][e.toLowerCase()]['responses'][String(code)] = {
                                        description,
                                        content: {
                                            'application/json': {
                                                schema: {
                                                    type: 'array',
                                                    items: {
                                                        type: 'object',
                                                        properties: typeResolver_1.getProperties(type.elementType.properties),
                                                    },
                                                },
                                            },
                                        },
                                    };
                                }
                                else if (type.properties) {
                                    responses[baseRoute + specificRoute][e.toLowerCase()]['responses'][String(code)] = {
                                        description,
                                        content: {
                                            'application/json': {
                                                schema: {
                                                    type: 'object',
                                                    properties: typeResolver_1.getProperties(type.properties),
                                                },
                                            },
                                        },
                                    };
                                }
                            }
                        }
                    }
                });
            }
        }
    }
    return responses;
}
function getFiles() {
    const paths = glob_1.default.sync(getBasePath() + 'src/**/*');
    const filtered = paths.filter((e) => e.endsWith('.ts') && !e.endsWith('.d.ts'));
    return filtered;
}
function getDecorators(node, name) {
    const decorators = node.decorators;
    if (!decorators || !decorators.length) {
        return [];
    }
    return decorators.filter((decorator) => {
        return (lodash_1.get(decorator, 'expression.expression.escapedText') === name || lodash_1.get(decorator, 'expression.escapedText') === name);
    });
}
function getDecoratorValues(decorator) {
    if (decorator && decorator.expression && Array.isArray(decorator.expression.arguments)) {
        return decorator.expression.arguments.map((e) => e.text);
    }
    return [];
}
